@@ -19,24 +19,23 @@ on_blue(red('hello'))+' '+on_red(blue('there'))+green('!')
 red('hello')
 """
 
-import itertools
 import re
-import sys
 from cwcwidth import wcswidth, wcwidth
+from itertools import chain
 from typing import (
-    Iterator,
-    Tuple,
-    List,
-    Union,
-    Optional,
     Any,
-    Mapping,
-    cast,
-    MutableMapping,
-    no_type_check,
-    Type,
     Callable,
+    Dict,
     Iterable,
+    Iterator,
+    List,
+    Mapping,
+    MutableMapping,
+    Optional,
+    Tuple,
+    Union,
+    cast,
+    no_type_check,
 )
 
 try:
@@ -70,14 +69,9 @@ two_arg_xforms: Mapping[str, Callable[[str, int], str]] = {
     "bg": lambda s, v: seq(v) + s + seq(RESET_BG),
 }
 
-# TODO unused, remove this in next major release
-xforms: MutableMapping[str, Union[Callable[[str], str], Callable[[str, int], str]]] = {}
-xforms.update(one_arg_xforms)
-xforms.update(two_arg_xforms)
 
-
-class FrozenDict(dict):
-    """Immutable dictionary class"""
+class FrozenAttributes(Dict[str, Union[int, bool]]):
+    """Immutable dictionary class for format string attributes"""
 
     @no_type_check
     def __setitem__(self, key, value):
@@ -87,11 +81,11 @@ class FrozenDict(dict):
     def update(self, *args, **kwds):
         raise Exception("Cannot change value.")
 
-    def extend(self, dictlike: Mapping[str, Union[int, bool]]) -> "FrozenDict":
-        return FrozenDict(itertools.chain(self.items(), dictlike.items()))
+    def extend(self, dictlike: Mapping[str, Union[int, bool]]) -> "FrozenAttributes":
+        return FrozenAttributes(chain(self.items(), dictlike.items()))
 
-    def remove(self, *keys: str) -> "FrozenDict":
-        return FrozenDict((k, v) for k, v in self.items() if k not in keys)
+    def remove(self, *keys: str) -> "FrozenAttributes":
+        return FrozenAttributes((k, v) for k, v in self.items() if k not in keys)
 
 
 def stable_format_dict(d: Mapping) -> str:
@@ -121,14 +115,14 @@ class Chunk:
         if not isinstance(string, str):
             raise ValueError("unicode string required, got %r" % string)
         self._s = string
-        self._atts = FrozenDict(atts if atts else {})
+        self._atts = FrozenAttributes(atts if atts else {})
 
     @property
     def s(self) -> str:
         return self._s
 
     @property
-    def atts(self) -> Mapping[str, Union[int, bool]]:
+    def atts(self) -> FrozenAttributes:
         "Attributes, e.g. {'fg': 34, 'bold': True} where 34 is the escape code for ..."
         return self._atts
 
@@ -137,7 +131,7 @@ class Chunk:
 
     @property
     def width(self) -> int:
-        width = wcswidth(self._s, None)
+        width = wcswidth(self._s)
         if len(self._s) > 0 and width < 1:
             raise ValueError("Can't calculate width of string %r" % self._s)
         return width
@@ -145,15 +139,15 @@ class Chunk:
     @cached_property
     def color_str(self) -> str:
         "Return an escape-coded string to write to the terminal."
-        s = self.s
-        for k, v in sorted(self.atts.items()):
+        s = self._s
+        for k, v in sorted(self._atts.items()):
             # (self.atts sorted for the sake of always acting the same.)
-            if k not in xforms:
+            if k not in one_arg_xforms and k not in two_arg_xforms:
                 # Unsupported SGR code
                 continue
             elif v is False:
                 continue
-            elif v is True:
+            elif k in one_arg_xforms:
                 s = one_arg_xforms[k](s)
             else:
                 s = two_arg_xforms[k](s, v)
@@ -168,16 +162,16 @@ class Chunk:
     def __eq__(self, other: Any) -> bool:
         if not isinstance(other, Chunk):
             return NotImplemented
-        return self.s == other.s and self.atts == other.atts
+        return self._s == other._s and self._atts == other._atts
 
     def __hash__(self) -> int:
-        return hash((self.s, self.atts))
+        return hash((self._s, self._atts))
 
     def __repr__(self) -> str:
         return "Chunk({s}{sep}{atts})".format(
-            s=repr(self.s),
-            sep=", " if self.atts else "",
-            atts=stable_format_dict(self.atts) if self.atts else "",
+            s=repr(self._s),
+            sep=", " if self._atts else "",
+            atts=stable_format_dict(self._atts) if self._atts else "",
         )
 
     def repr_part(self) -> str:
@@ -191,10 +185,10 @@ class Chunk:
             else:
                 return att
 
-        atts_out = {k: v for (k, v) in self.atts.items() if v}
+        atts_out = {k: v for (k, v) in self._atts.items() if v}
         return (
             "".join(pp_att(att) + "(" for att in sorted(atts_out))
-            + repr(self.s)
+            + repr(self._s)
             + ")" * len(atts_out)
         )
 
@@ -296,8 +290,8 @@ class FmtStr:
         self._s: Optional[str] = None
         self._width: Optional[int] = None
 
-    @classmethod
-    def from_str(cls, s: str) -> "FmtStr":
+    @staticmethod
+    def from_str(s: str) -> "FmtStr":
         r"""
         Return a FmtStr representing input.
 
@@ -416,8 +410,9 @@ class FmtStr:
     def copy_with_new_atts(self, **attributes: Union[bool, int]) -> "FmtStr":
         """Returns a new FmtStr with the same content but new formatting"""
 
-        result = FmtStr(*(Chunk(bfs.s, bfs.atts.extend(attributes)) for bfs in self.chunks))  # type: ignore
-        return result
+        return FmtStr(
+            *(Chunk(bfs.s, bfs.atts.extend(attributes)) for bfs in self.chunks)
+        )
 
     def join(self, iterable: Iterable[Union[str, "FmtStr"]]) -> "FmtStr":
         """Joins an iterable yielding strings or FmtStrs with self as separator"""
@@ -456,8 +451,8 @@ class FmtStr:
         return [
             self[start:end]
             for start, end in zip(
-                [0] + [m.end() for m in matches],
-                [m.start() for m in matches] + [len(s)],
+                chain((0,), (m.end() for m in matches)),
+                chain((m.start() for m in matches), (len(s),)),
             )
         ]
 
@@ -828,7 +823,7 @@ def normalize_slice(length: int, index: Union[int, slice]) -> slice:
 
 
 def parse_args(
-    args: Tuple[Union[bytes, str], ...],
+    args: Tuple[str, ...],
     kwargs: MutableMapping[str, Union[int, bool, str]],
 ) -> Mapping[str, Union[int, bool]]:
     """Returns a kwargs dictionary by turning args into kwargs"""
@@ -836,9 +831,8 @@ def parse_args(
         args += (cast(str, kwargs["style"]),)
         del kwargs["style"]
     for arg in args:
-        arg = cast(str, arg)
-        if not isinstance(arg, (bytes, str)):
-            raise ValueError("args must be strings:" + repr(args))
+        if not isinstance(arg, str):
+            raise ValueError(f"args must be strings: {arg!r}")
         if arg.lower() in FG_COLORS:
             if "fg" in kwargs:
                 raise ValueError("fg specified twice")
@@ -850,20 +844,20 @@ def parse_args(
         elif arg.lower() in STYLES:
             kwargs[arg] = True
         else:
-            raise ValueError("couldn't process arg: " + repr(arg))
+            raise ValueError(f"couldn't process arg: {args!r}")
     for k in kwargs:
-        if k not in ["fg", "bg"] + list(STYLES.keys()):
+        if k not in ("fg", "bg") and k not in STYLES.keys():
             raise ValueError("Can't apply that transformation")
     if "fg" in kwargs:
         if kwargs["fg"] in FG_COLORS:
             kwargs["fg"] = FG_COLORS[cast(str, kwargs["fg"])]
         if kwargs["fg"] not in list(FG_COLORS.values()):
-            raise ValueError("Bad fg value: %r" % kwargs["fg"])
+            raise ValueError(f"Bad fg value: {kwargs['fg']!r}")
     if "bg" in kwargs:
         if kwargs["bg"] in BG_COLORS:
             kwargs["bg"] = BG_COLORS[cast(str, kwargs["bg"])]
         if kwargs["bg"] not in list(BG_COLORS.values()):
-            raise ValueError("Bad bg value: %r" % kwargs["bg"])
+            raise ValueError(f"Bad bg value: {kwargs['bg']!r}")
     return cast(MutableMapping[str, Union[int, bool]], kwargs)
 
 
@@ -877,14 +871,10 @@ def fmtstr(string: Union[str, FmtStr], *args: Any, **kwargs: Any) -> FmtStr:
     on_red(bold(blue('blarg')))
     """
     atts = parse_args(args, kwargs)
-    if isinstance(string, FmtStr):
-        pass
-    elif isinstance(string, (bytes, str)):
+    if isinstance(string, str):
         string = FmtStr.from_str(string)
-    else:
+    elif not isinstance(string, FmtStr):
         raise ValueError(
-            "Bad Args: {!r} (of type {}), {!r}, {!r}".format(
-                string, type(string), args, kwargs
-            )
+            f"Bad Args: {string!r} (of type {type(string)}), {args!r}, {kwargs!r}"
         )
     return string.copy_with_new_atts(**atts)
