@@ -1,5 +1,3 @@
-from __future__ import unicode_literals
-
 import sys
 import signal
 import logging
@@ -38,7 +36,7 @@ def array_size_test(window):
                     w.scroll_down()
                 elif isinstance(c, events.WindowChangeEvent):
                     a = w.array_from_text("window just changed to %d rows and %d columns" % (c.rows, c.columns))
-                elif c == u'<ESC>': # allows exit without keyboard interrupt
+                elif c == '<ESC>': # allows exit without keyboard interrupt
                     break
                 elif c == '\x0c': # ctrl-L
                     [w.write('\n') for _ in range(rows)]
